@@ -19,7 +19,41 @@
 #ifndef __G_LIB_H__
 #define __G_LIB_H__
 
-/* system specific config file
+/* system specific config file glibconfig.h provides definitions for
+ * the extrema of many of the standard types. These are:
+ *
+ *  G_MINSHORT, G_MAXSHORT
+ *  G_MININT, G_MAXINT
+ *  G_MINLONG, G_MAXLONG
+ *  G_MINFLOAT, G_MAXFLOAT
+ *  G_MINDOUBLE, G_MAXDOUBLE
+ *
+ * It also provides the following typedefs:
+ *
+ *  gint8, guint8
+ *  gint16, guint16
+ *  gint32, guint32
+ *  gint64, guint64
+ *
+ * It defines the G_BYTE_ORDER symbol to one of G_*_ENDIAN (see later in
+ * this file). 
+ *
+ * And it provides a way to store and retrieve a `gint' in/from a `gpointer'.
+ * This is useful to pass an integer instead of a pointer to a callback.
+ *
+ *  GINT_TO_POINTER(i), GUINT_TO_POINTER(i)
+ *  GPOINTER_TO_INT(p), GPOINTER_TO_UINT(p)
+ *
+ * Finally, it provide the following wrappers to STDC functions:
+ *
+ *  g_ATEXIT
+ *    To register hooks which are executed on exit().
+ *    Usually a wrapper for STDC atexit.
+ *
+ *  void *g_memmove(void *dest, const void *src, guint count);
+ *    A wrapper for STDC memmove, or an implementation, if memmove doesn't
+ *    exist.  The prototype looks like the above, give or take a const,
+ *    or size_t.
  */
 #include <glibconfig.h>
 
@@ -34,55 +68,52 @@
 #endif
 
 
-/* glib provides definitions for the extrema of many
- *  of the standard types. These are:
- * G_MINFLOAT
- * G_MAXFLOAT
- * G_MINDOUBLE
- * G_MAXDOUBLE
- * G_MINSHORT
- * G_MAXSHORT
- * G_MININT
- * G_MAXINT
- * G_MINLONG
- * G_MAXLONG
- *
- * We include limits.h before float.h to work around a egcs 1.1
- * oddity on Solaris 2.5.1
- */
-#ifdef HAVE_LIMITS_H
-#  include <limits.h>
-#  define G_MINSHORT  SHRT_MIN
-#  define G_MAXSHORT  SHRT_MAX
-#  define G_MININT    INT_MIN
-#  define G_MAXINT    INT_MAX
-#  define G_MINLONG   LONG_MIN
-#  define G_MAXLONG   LONG_MAX
-#elif HAVE_VALUES_H
-#  ifdef HAVE_FLOAT_H
-#    include <values.h>
-#  endif /* HAVE_FLOAT_H */
-#  define G_MINSHORT  MINSHORT
-#  define G_MAXSHORT  MAXSHORT
-#  define G_MININT    MININT
-#  define G_MAXINT    MAXINT
-#  define G_MINLONG   MINLONG
-#  define G_MAXLONG   MAXLONG
-#endif /* HAVE_VALUES_H */
+#ifdef NATIVE_WIN32
 
-#ifdef HAVE_FLOAT_H
-#  include <float.h>
-#  define G_MINFLOAT   FLT_MIN
-#  define G_MAXFLOAT   FLT_MAX
-#  define G_MINDOUBLE  DBL_MIN
-#  define G_MAXDOUBLE  DBL_MAX
-#elif HAVE_VALUES_H
-#  include <values.h>
-#  define G_MINFLOAT  MINFLOAT
-#  define G_MAXFLOAT  MAXFLOAT
-#  define G_MINDOUBLE MINDOUBLE
-#  define G_MAXDOUBLE MAXDOUBLE
-#endif /* HAVE_VALUES_H */
+/* On native Win32, directory separator is the backslash, and search path
+ * separator is the semicolon.
+ */
+#define G_DIR_SEPARATOR '\\'
+#define G_DIR_SEPARATOR_S "\\"
+#define G_SEARCHPATH_SEPARATOR ';'
+#define G_SEARCHPATH_SEPARATOR_S ";"
+
+#else  /* !NATIVE_WIN32 */
+
+/* Unix */
+
+#define G_DIR_SEPARATOR '/'
+#define G_DIR_SEPARATOR_S "/"
+#define G_SEARCHPATH_SEPARATOR ':'
+#define G_SEARCHPATH_SEPARATOR_S ":"
+
+#endif /* !NATIVE_WIN32 */
+
+#ifdef _MSC_VER
+/* Make MSVC more pedantic, this is a recommended pragma list
+ * from _Win32_Programming_ by Rector and Newcomer.
+ */
+#pragma warning(error:4002)
+#pragma warning(error:4003)
+#pragma warning(1:4010)
+#pragma warning(error:4013)
+#pragma warning(1:4016)
+#pragma warning(error:4020)
+#pragma warning(error:4021)
+#pragma warning(error:4027)
+#pragma warning(error:4029)
+#pragma warning(error:4033)
+#pragma warning(error:4035)
+#pragma warning(error:4045)
+#pragma warning(error:4047)
+#pragma warning(error:4049)
+#pragma warning(error:4053)
+#pragma warning(error:4071)
+#pragma warning(disable:4101)
+#pragma warning(error:4150)
+
+#pragma warning(disable:4244)	/* No possible loss of data warnings, please */
+#endif /* _MSC_VER */
 
 
 #ifdef __cplusplus
@@ -134,20 +165,15 @@ extern "C" {
 #endif /* !G_VA_COPY */
 
 
-/* Provide simple enum value macro wrappers that ease automated
- * enum value stringification code. [abandoned]
+/* Provide convenience macros for handling structure
+ * fields through their offsets.
  */
-#if	!defined (G_CODE_GENERATION)
-#define G_ENUM( EnumerationName )		EnumerationName
-#define G_FLAGS( EnumerationName )		EnumerationName
-#define G_NV( VALUE_NAME , value_nick, VALUE)	VALUE_NAME = (VALUE)
-#define G_SV( VALUE_NAME, value_nick )		VALUE_NAME
-#else	/* G_CODE_GENERATION */
-#define G_ENUM( EnumerationName )		G_ENUM_E + EnumerationName +
-#define G_FLAGS( EnumerationName )		G_ENUM_F + EnumerationName +
-#define G_NV( VALUE_NAME , value_nick, VALUE)	G_ENUM_V + VALUE_NAME + value_nick +
-#define G_SV( VALUE_NAME, value_nick )		G_ENUM_V + VALUE_NAME + value_nick +
-#endif	/* G_CODE_GENERATION */
+#define G_STRUCT_OFFSET(struct_type, member)	\
+    ((gulong) ((gchar*) &((struct_type*) 0)->member))
+#define G_STRUCT_MEMBER_P(struct_p, struct_offset)   \
+    ((gpointer) ((gchar*) (struct_p) + (gulong) (struct_offset)))
+#define G_STRUCT_MEMBER(member_type, struct_p, struct_offset)   \
+    (*(member_type*) G_STRUCT_MEMBER_P ((struct_p), (struct_offset)))
 
 
 /* inlining hassle. for compilers that don't allow the `inline' keyword,
@@ -262,34 +288,24 @@ extern "C" {
 #define	G_GNUC_PRETTY_FUNCTION	("")
 #endif	/* !__GNUC__ */
 
-
 /* we try to provide a usefull equivalent for ATEXIT if it is
  * not defined, but use is actually abandoned. people should
  * use g_atexit() instead.
- * keep this in sync with gutils.c.
  */
 #ifndef ATEXIT
-#  ifdef HAVE_ATEXIT
-#    ifdef NeXT /* @#%@! NeXTStep */
-#      define ATEXIT(proc)   (!atexit (proc))
-#    else /* !NeXT */
-#      define ATEXIT(proc)   (atexit (proc))
-#    endif /* !NeXT */
-#  elif defined (HAVE_ON_EXIT)
-#    define ATEXIT(proc)   (on_exit ((void (*)(int, void *))(proc), NULL))
-#  else
-#  error Could not determine proper atexit() implementation
-#  endif
+# define ATEXIT(proc)	g_ATEXIT(proc)
 #else
-#  define G_NATIVE_ATEXIT
+# define G_NATIVE_ATEXIT
 #endif /* ATEXIT */
 
-/* Hacker macro to place breakpoints for x86 machines.
+/* Hacker macro to place breakpoints for elected machines.
  * Actual use is strongly deprecated of course ;)
  */
-#if	defined (__i386__) && defined (__GNUC__)
-#define	G_BREAKPOINT()		G_STMT_START{ __asm__ volatile ("int $03"); }G_STMT_END
-#else	/* !__i386__ */
+#if defined (__i386__) && defined (__GNUC__) && __GNUC__ >= 2
+#define	G_BREAKPOINT()		G_STMT_START{ __asm__ __volatile__ ("int $03"); }G_STMT_END
+#elif defined (__alpha__) && defined (__GNUC__) && __GNUC__ >= 2
+#define	G_BREAKPOINT()		G_STMT_START{ __asm__ __volatile__ ("bpt"); }G_STMT_END
+#else	/* !__i386__ && !__alpha__ */
 #define	G_BREAKPOINT()
 #endif	/* __i386__ */
 
@@ -455,54 +471,6 @@ extern "C" {
 #endif /* !G_DISABLE_CHECKS */
 
 
-/* Portable endian checks and conversions
- */
-
-#define G_LITTLE_ENDIAN 1234
-#define G_BIG_ENDIAN    4321
-#define G_PDP_ENDIAN    3412	/	/* unused, need specific PDP check */	
-
-#ifdef WORDS_BIGENDIAN
-#define G_BYTE_ORDER G_BIG_ENDIAN
-#else
-#define G_BYTE_ORDER G_LITTLE_ENDIAN
-#endif
-
-#define GULONG_SWAP_LE_BE(long_val)     (((gulong) \
-    (((gulong) (long_val)) & 0x000000ffU) << 24) | \
-    (((gulong) (long_val)) & 0x0000ff00U) <<  8) | \
-    (((gulong) (long_val)) & 0x00ff0000U) >>  8) | \
-    (((gulong) (long_val)) & 0xff000000U) >> 24)))
-#define GULONG_SWAP_LE_PDP(long_val)    (((gulong) \
-    (((gulong) (long_val)) & 0x0000ffffU) << 16) | \
-    (((gulong) (long_val)) & 0xffff0000U) >> 16)))
-#define GULONG_SWAP_BE_PDP(long_val)    (((gulong) \
-    (((gulong) (long_val)) & 0x000000ffU) << 8) | \
-    (((gulong) (long_val)) & 0x0000ff00U) >> 8) | \
-    (((gulong) (long_val)) & 0x00ff0000U) << 8) | \
-    (((gulong) (long_val)) & 0xff000000U) >> 8)))
-
-#if     G_BYTE_ORDER == G_LITTLE_ENDIAN
-#  define GLONG_TO_LE(long_val)         ((glong) (long_val))
-#  define GULONG_TO_LE(long_val)        ((gulong) (long_val))
-#  define GLONG_TO_BE(long_val)         ((glong) GULONG_SWAP_LE_BE (long_val))
-#  define GULONG_TO_BE(long_val)        (GULONG_SWAP_LE_BE (long_val))
-#  define GLONG_FROM_LE(long_val)       ((glong) (long_val))
-#  define GULONG_FROM_LE(long_val)      ((gulong) (long_val))
-#  define GLONG_FROM_BE(long_val)       ((glong) GULONG_SWAP_LE_BE (long_val))
-#  define GULONG_FROM_BE(long_val)      (GULONG_SWAP_LE_BE (long_val))
-#elif   G_BYTE_ORDER == G_BIG_ENDIAN
-#  define GLONG_TO_LE(long_val)         ((glong) GULONG_SWAP_LE_BE (long_val))
-#  define GULONG_TO_LE(long_val)        (GULONG_SWAP_LE_BE (long_val))
-#  define GLONG_TO_BE(long_val)         ((glong) (long_val))
-#  define GULONG_TO_BE(long_val)        ((gulong) (long_val))
-#  define GLONG_FROM_LE(long_val)       ((glong) GULONG_SWAP_LE_BE (long_val))
-#  define GULONG_FROM_LE(long_val)      (GULONG_SWAP_LE_BE (long_val))
-#  define GLONG_FROM_BE(long_val)       ((glong) (long_val))
-#  define GULONG_FROM_BE(long_val)      ((gulong) (long_val))
-#endif
-
-
 /* Provide type definitions for commonly used types.
  *  These are useful because a "gint8" can be adjusted
  *  to be 1 byte (8 bits) on all platforms. Similarly and
@@ -538,59 +506,6 @@ typedef double gldouble;
 typedef void* gpointer;
 typedef const void *gconstpointer;
 
-#if (SIZEOF_CHAR == 1)
-typedef signed char	gint8;
-typedef unsigned char	guint8;
-#endif /* SIZEOF_CHAR */
-
-#if (SIZEOF_SHORT == 2)
-typedef signed short	gint16;
-typedef unsigned short	guint16;
-#endif /* SIZEOF_SHORT */
-
-#if (SIZEOF_INT == 4)
-typedef signed int	gint32;
-typedef unsigned int	guint32;
-#elif (SIZEOF_LONG == 4)
-typedef signed long	gint32;
-typedef unsigned long	guint32;
-#endif /* SIZEOF_INT */
-
-#if (SIZEOF_LONG == 8)
-#define HAVE_GINT64 1
-typedef signed long gint64;
-typedef unsigned long guint64;
-#elif (SIZEOF_LONG_LONG == 8)
-#define HAVE_GINT64 1
-typedef signed long long gint64;
-typedef unsigned long long guint64;
-#else
-/* No gint64 */
-#undef HAVE_GINT64
-#endif
-
-
-/* Define macros for storing integers inside pointers
- */
-#if (SIZEOF_INT == SIZEOF_VOID_P)
-
-#define GPOINTER_TO_INT(p) ((gint)(p))
-#define GPOINTER_TO_UINT(p) ((guint)(p))
-
-#define GINT_TO_POINTER(i) ((gpointer)(i))
-#define GUINT_TO_POINTER(u) ((gpointer)(u))
-
-#elif (SIZEOF_LONG == SIZEOF_VOID_P)
-
-#define GPOINTER_TO_INT(p) ((gint)(glong)(p))
-#define GPOINTER_TO_UINT(p) ((guint)(gulong)(p))
-
-#define GINT_TO_POINTER(i) ((gpointer)(glong)(i))
-#define GUINT_TO_POINTER(u) ((gpointer)(gulong)(u))
-
-#else
-#error SIZEOF_VOID_P unknown - This should never happen
-#endif
 
 typedef gint32	gssize;
 typedef guint32 gsize;
@@ -598,14 +513,184 @@ typedef guint32 GQuark;
 typedef gint32	GTime;
 
 
-/* Glib version.
+/* Portable endian checks and conversions
  */
-extern const guint glib_major_version;
-extern const guint glib_minor_version;
-extern const guint glib_micro_version;
-extern const guint glib_interface_age;
-extern const guint glib_binary_age;
 
+#define G_LITTLE_ENDIAN 1234
+#define G_BIG_ENDIAN    4321
+#define G_PDP_ENDIAN    3412		/* unused, need specific PDP check */	
+
+/* Basic bit swapping functions
+ */
+#define GUINT16_SWAP_LE_BE_CONSTANT(val)	((guint16) ( \
+    (((guint16) (val) & (guint16) 0x00ffU) << 8) | \
+    (((guint16) (val) & (guint16) 0xff00U) >> 8)))
+#define GUINT32_SWAP_LE_BE_CONSTANT(val)	((guint32) ( \
+    (((guint32) (val) & (guint32) 0x000000ffU) << 24) | \
+    (((guint32) (val) & (guint32) 0x0000ff00U) <<  8) | \
+    (((guint32) (val) & (guint32) 0x00ff0000U) >>  8) | \
+    (((guint32) (val) & (guint32) 0xff000000U) >> 24)))
+
+/* Intel specific stuff for speed
+ */
+#if defined (__i386__) && defined (__GNUC__) && __GNUC__ >= 2
+
+#  define GUINT16_SWAP_LE_BE_X86(val) \
+     (__extension__					\
+      ({ register guint16 __v;				\
+	 if (__builtin_constant_p (val))		\
+	   __v = GUINT16_SWAP_LE_BE_CONSTANT (val);	\
+	 else						\
+	   __asm__ __const__ ("rorw $8, %w0"		\
+			      : "=r" (__v)		\
+			      : "0" ((guint16) (val)));	\
+	__v; }))
+
+#  define GUINT16_SWAP_LE_BE(val) (GUINT16_SWAP_LE_BE_X86 (val))
+
+#  if !defined(__i486__) && !defined(__i586__) \
+      && !defined(__pentium__) && !defined(__i686__) && !defined(__pentiumpro__)
+#     define GUINT32_SWAP_LE_BE_X86(val) \
+        (__extension__						\
+         ({ register guint32 __v;				\
+	    if (__builtin_constant_p (val))			\
+	      __v = GUINT32_SWAP_LE_BE_CONSTANT (val);		\
+	  else							\
+	    __asm__ __const__ ("rorw $8, %w0\n\t"		\
+			       "rorl $16, %0\n\t"		\
+			       "rorw $8, %w0"			\
+			       : "=r" (__v)			\
+			       : "0" ((guint32) (val)));	\
+	__v; }))
+
+#  else /* 486 and higher has bswap */
+#     define GUINT32_SWAP_LE_BE_X86(val) \
+        (__extension__						\
+         ({ register guint32 __v;				\
+	    if (__builtin_constant_p (val))			\
+	      __v = GUINT32_SWAP_LE_BE_CONSTANT (val);		\
+	  else							\
+	    __asm__ __const__ ("bswap %0"			\
+			       : "=r" (__v)			\
+			       : "0" ((guint32) (val)));	\
+	__v; }))
+#  endif /* processor specific 32-bit stuff */
+
+#  define GUINT32_SWAP_LE_BE(val) (GUINT32_SWAP_LE_BE_X86 (val))
+
+#else /* !__i386__ */
+#  define GUINT16_SWAP_LE_BE(val) (GUINT16_SWAP_LE_BE_CONSTANT (val))
+#  define GUINT32_SWAP_LE_BE(val) (GUINT32_SWAP_LE_BE_CONSTANT (val))
+#endif /* __i386__ */
+
+#ifdef G_HAVE_GINT64
+#  define GUINT64_SWAP_LE_BE_CONSTANT(val)	((guint64) ( \
+      (((guint64) (val) &						\
+	(guint64) G_GINT64_CONSTANT(0x00000000000000ffU)) << 56) |	\
+      (((guint64) (val) &						\
+	(guint64) G_GINT64_CONSTANT(0x000000000000ff00U)) << 40) |	\
+      (((guint64) (val) &						\
+	(guint64) G_GINT64_CONSTANT(0x0000000000ff0000U)) << 24) |	\
+      (((guint64) (val) &						\
+	(guint64) G_GINT64_CONSTANT(0x00000000ff000000U)) <<  8) |	\
+      (((guint64) (val) &						\
+	(guint64) G_GINT64_CONSTANT(0x000000ff00000000U)) >>  8) |	\
+      (((guint64) (val) &						\
+	(guint64) G_GINT64_CONSTANT(0x0000ff0000000000U)) >> 24) |	\
+      (((guint64) (val) &						\
+	(guint64) G_GINT64_CONSTANT(0x00ff000000000000U)) >> 40) |	\
+      (((guint64) (val) &						\
+	(guint64) G_GINT64_CONSTANT(0xff00000000000000U)) >> 56)))
+
+#  if defined (__i386__) && defined (__GNUC__) && __GNUC__ >= 2
+#    define GUINT64_SWAP_LE_BE_X86(val) \
+	(__extension__						\
+	 ({ union { guint64 __ll;				\
+		    guint32 __l[2]; } __r;			\
+	    if (__builtin_constant_p (val))			\
+	      __r.__ll = GUINT64_SWAP_LE_BE_CONSTANT (val);	\
+	    else						\
+	      {							\
+	 	union { guint64 __ll;				\
+			guint32 __l[2]; } __w;			\
+		__w.__ll = ((guint64) val);			\
+		__r.__l[0] = GUINT32_SWAP_LE_BE (__w.__l[1]);	\
+		__r.__l[1] = GUINT32_SWAP_LE_BE (__w.__l[0]);	\
+	      }							\
+	  __r.__ll; }))
+
+#    define GUINT64_SWAP_LE_BE(val) (GUINT64_SWAP_LE_BE_X86 (val))
+
+#  else /* !__i386__ */
+#    define GUINT64_SWAP_LE_BE(val) (GUINT64_SWAP_LE_BE_CONSTANT(val))
+#  endif
+#endif
+
+#define GUINT16_SWAP_LE_PDP(val)	((guint16) (val))
+#define GUINT16_SWAP_BE_PDP(val)	(GUINT16_SWAP_LE_BE (val))
+#define GUINT32_SWAP_LE_PDP(val)	((guint32) ( \
+    (((guint32) (val) & (guint32) 0x0000ffffU) << 16) | \
+    (((guint32) (val) & (guint32) 0xffff0000U) >> 16)))
+#define GUINT32_SWAP_BE_PDP(val)	((guint32) ( \
+    (((guint32) (val) & (guint32) 0x00ff00ffU) << 8) | \
+    (((guint32) (val) & (guint32) 0xff00ff00U) >> 8)))
+
+/* The TO_?E stuff is defined in glibconfig.h. The transformation is symmetric,
+   so the FROM just maps to the TO.
+ */
+#define GINT16_FROM_LE(val)	(GINT16_TO_LE (val))
+#define GUINT16_FROM_LE(val)	(GUINT16_TO_LE (val))
+#define GINT16_FROM_BE(val)	(GINT16_TO_BE (val))
+#define GUINT16_FROM_BE(val)	(GUINT16_TO_BE (val))
+#define GINT32_FROM_LE(val)	(GINT32_TO_LE (val))
+#define GUINT32_FROM_LE(val)	(GUINT32_TO_LE (val))
+#define GINT32_FROM_BE(val)	(GINT32_TO_BE (val))
+#define GUINT32_FROM_BE(val)	(GUINT32_TO_BE (val))
+
+#ifdef G_HAVE_GINT64
+#define GINT64_FROM_LE(val)	(GINT32_TO_LE (val))
+#define GUINT64_FROM_LE(val)	(GUINT32_TO_LE (val))
+#define GINT64_FROM_BE(val)	(GINT32_TO_BE (val))
+#define GUINT64_FROM_BE(val)	(GUINT32_TO_BE (val))
+#endif
+
+#define GLONG_FROM_LE(val)	(GLONG_TO_LE (val))
+#define GULONG_FROM_LE(val)	(GULONG_TO_LE (val))
+#define GLONG_FROM_BE(val)	(GLONG_TO_BE (val))
+#define GULONG_FROM_BE(val)	(GULONG_TO_BE (val))
+
+#define GINT_FROM_LE(val)	(GINT_TO_LE (val))
+#define GUINT_FROM_LE(val)	(GUINT_TO_LE (val))
+#define GINT_FROM_BE(val)	(GINT_TO_BE (val))
+#define GUINT_FROM_BE(val)	(GUINT_TO_BE (val))
+
+/* Portable versions of host-network order stuff
+ */
+#define g_ntohl(val) (GUINT32_FROM_BE (val))
+#define g_ntohs(val) (GUINT16_FROM_BE (val))
+#define g_htonl(val) (GUINT32_TO_BE (val))
+#define g_htons(val) (GUINT16_TO_BE (val))
+
+
+/* Glib version.
+ * we prefix variable declarations so they can
+ * properly get exported in windows dlls.
+ */
+#ifdef NATIVE_WIN32
+#  ifdef GLIB_COMPILATION
+#    define GUTILS_C_VAR __declspec(dllexport)
+#  else /* !GLIB_COMPILATION */
+#    define GUTILS_C_VAR __declspec(dllimport)
+#  endif /* !GLIB_COMPILATION */
+#else /* !NATIVE_WIN32 */
+#  define GUTILS_C_VAR extern
+#endif /* !NATIVE_WIN32 */
+
+GUTILS_C_VAR const guint glib_major_version;
+GUTILS_C_VAR const guint glib_minor_version;
+GUTILS_C_VAR const guint glib_micro_version;
+GUTILS_C_VAR const guint glib_interface_age;
+GUTILS_C_VAR const guint glib_binary_age;
 
 /* Forward declarations of glib types.
  */
@@ -634,6 +719,7 @@ typedef struct _GTimer		GTimer;
 typedef struct _GTree		GTree;
 typedef struct _GTuples		GTuples;
 typedef union  _GTokenValue	GTokenValue;
+typedef struct _GIOChannel	GIOChannel;
 
 
 typedef enum
@@ -707,6 +793,8 @@ typedef void		(*GHookMarshaller)	(GHook		*hook,
 						 gpointer	 data);
 typedef void		(*GHookFunc)		(gpointer	 data);
 typedef gboolean	(*GHookCheckFunc)	(gpointer	 data);
+typedef void		(*GHookFreeFunc)	(GHookList      *hook_list,
+						 GHook          *hook);
 typedef void		(*GLogFunc)		(const gchar   *log_domain,
 						 GLogLevelFlags	log_level,
 						 const gchar   *message,
@@ -774,13 +862,6 @@ struct _GDebugKey
   guint	 value;
 };
 
-struct _GCache { gint dummy; };
-struct _GTree { gint dummy; };
-struct _GTimer { gint dummy; };
-struct _GMemChunk { gint dummy; };
-struct _GListAllocator { gint dummy; };
-struct _GStringChunk { gint dummy; };
-
 
 /* Doubly linked lists
  */
@@ -821,6 +902,8 @@ guint  g_list_length		(GList		*list);
 void   g_list_foreach		(GList		*list,
 				 GFunc		 func,
 				 gpointer	 user_data);
+GList* g_list_sort              (GList          *list,
+		                 GCompareFunc    compare_func);
 gpointer g_list_nth_data	(GList		*list,
 				 guint		 n);
 #define g_list_previous(list)	((list) ? (((GList *)(list))->prev) : NULL)
@@ -865,6 +948,8 @@ guint	g_slist_length		(GSList		*list);
 void	g_slist_foreach		(GSList		*list,
 				 GFunc		 func,
 				 gpointer	 user_data);
+GSList*  g_slist_sort           (GSList          *list,
+		                 GCompareFunc    compare_func);
 gpointer g_slist_nth_data	(GSList		*list,
 				 guint		 n);
 #define g_slist_next(slist)	((slist) ? (((GSList *)(slist))->next) : NULL)
@@ -1064,6 +1149,7 @@ struct _GHookList
   guint		 is_setup : 1;
   GHook		*hooks;
   GMemChunk	*hook_memchunk;
+  GHookFreeFunc	 hook_free; /* virtual function */
 };
 
 struct _GHook
@@ -1172,7 +1258,7 @@ void g_on_error_stack_trace (const gchar *prg_name);
 
 /* Logging mechanism
  */
-extern const gchar			*g_log_domain_glib;
+extern	        const gchar		*g_log_domain_glib;
 guint		g_log_set_handler	(const gchar	*log_domain,
 					 GLogLevelFlags	 log_levels,
 					 GLogFunc	 log_func,
@@ -1347,11 +1433,12 @@ gchar*	 g_strerror		(gint	      errnum);
 gchar*	 g_strsignal		(gint	      signum);
 gint	 g_strcasecmp		(const gchar *s1,
 				 const gchar *s2);
+gint	 g_strncasecmp		(const gchar *s1,
+				 const gchar *s2,
+				 guint 	      n);
 void	 g_strdown		(gchar	     *string);
 void	 g_strup		(gchar	     *string);
 void	 g_strreverse		(gchar	     *string);
-gpointer g_memdup		(gconstpointer mem,
-				 guint	       byte_size);
 /* removes leading spaces */
 gchar*   g_strchug              (gchar        *string);
 /* removes trailing spaces */
@@ -1375,6 +1462,9 @@ gchar*	 g_strconcat		(const gchar *string1,
 				 ...); /* NULL terminated */
 gchar*   g_strjoin		(const gchar  *separator,
 				 ...); /* NULL terminated */
+gchar*	 g_strescape		(gchar	      *string);
+gpointer g_memdup		(gconstpointer mem,
+				 guint	       byte_size);
 
 /* NULL terminated string arrays.
  * g_strsplit() splits up string into max_tokens tokens at delim and
@@ -1422,25 +1512,16 @@ gint	g_vsnprintf		(gchar	     *string,
 				 gchar const *format,
 				 va_list      args);
 gchar*	g_basename		(const gchar *file_name);
+/* Check if a file name is an absolute path */
+gboolean g_path_is_absolute	(const gchar *file_name);
+/* In case of absolute paths, skip the root part */
+gchar*  g_path_skip_root	(gchar       *file_name);
 
 /* strings are newly allocated with g_malloc() */
 gchar*	g_dirname		(const gchar *file_name);
 gchar*	g_get_current_dir	(void);
+gchar*  g_getenv		(const gchar *variable);
 
-/* We make the assumption that if memmove isn't available, then
- * bcopy will do the job. This isn't safe everywhere. (bcopy can't
- * necessarily handle overlapping copies).
- * Either way, g_memmove() will not return a value.
- */
-#ifdef HAVE_MEMMOVE
-#define g_memmove(dest, src, size)	G_STMT_START {	\
-     memmove ((dest), (src), (size));			\
-} G_STMT_END
-#else
-#define g_memmove(dest, src, size)	G_STMT_START {	\
-     bcopy ((src), (dest), (size));			\
-} G_STMT_END
-#endif
 
 /* we use a GLib function as a replacement for ATEXIT, so
  * the programmer is not required to check the return value
@@ -1478,7 +1559,7 @@ g_bit_nth_msf (guint32 mask,
 	       gint    nth_bit)
 {
   if (nth_bit < 0)
-    nth_bit = 33;
+    nth_bit = 32;
   do
     {
       nth_bit--;
@@ -1507,7 +1588,6 @@ g_bit_storage (guint number)
 }
 #endif	/* G_CAN_INLINE */
 
-
 /* String Chunks
  */
 GStringChunk* g_string_chunk_new	   (gint size);
@@ -1555,30 +1635,37 @@ void	 g_string_sprintfa  (GString	 *string,
 			     ...) G_GNUC_PRINTF (2, 3);
 
 
-/* Resizable arrays
+/* Resizable arrays, remove fills any cleared spot and shortens the
+ * array, while preserving the order. remove_fast will distort the
+ * order by moving the last element to the position of the removed 
  */
 
 #define g_array_append_val(a,v) g_array_append_vals(a,&v,1)
 #define g_array_prepend_val(a,v) g_array_prepend_vals(a,&v,1)
 #define g_array_index(a,t,i) (((t*)a->data)[i])
 
-GArray* g_array_new	     (gboolean	    zero_terminated,
-			      gboolean	    clear,
-			      guint	    element_size);
-void	g_array_free	     (GArray	   *array,
-			      gboolean	    free_segment);
-GArray* g_array_append_vals  (GArray	   *array,
-			      gconstpointer data,
-			      guint	    len);
-GArray* g_array_prepend_vals (GArray	   *array,
-			      gconstpointer data,
-			      guint	    len);
-GArray* g_array_set_size     (GArray	   *array,
-			      guint	    length);
+GArray* g_array_new	          (gboolean	    zero_terminated,
+				   gboolean	    clear,
+				   guint	    element_size);
+void	g_array_free	          (GArray	   *array,
+				   gboolean	    free_segment);
+GArray* g_array_append_vals       (GArray	   *array,
+				   gconstpointer    data,
+				   guint	    len);
+GArray* g_array_prepend_vals      (GArray	   *array,
+				   gconstpointer    data,
+				   guint	    len);
+GArray* g_array_set_size          (GArray	   *array,
+				   guint	    length);
+GArray* g_array_remove_index	  (GArray	   *array,
+				   guint	    index);
+GArray* g_array_remove_index_fast (GArray	   *array,
+				   guint	    index);
 
 /* Resizable pointer array.  This interface is much less complicated
  * than the above.  Add appends appends a pointer.  Remove fills any
- * cleared spot and shortens the array.
+ * cleared spot and shortens the array. remove_fast will again distort
+ * order.  
  */
 #define	    g_ptr_array_index(array,index) (array->pdata)[index]
 GPtrArray*  g_ptr_array_new		   (void);
@@ -1587,9 +1674,13 @@ void	    g_ptr_array_free		   (GPtrArray	*array,
 void	    g_ptr_array_set_size	   (GPtrArray	*array,
 					    gint	 length);
 gpointer    g_ptr_array_remove_index	   (GPtrArray	*array,
-					    gint	 index);
+					    guint	 index);
+gpointer    g_ptr_array_remove_index_fast  (GPtrArray	*array,
+					    guint	 index);
 gboolean    g_ptr_array_remove		   (GPtrArray	*array,
 					    gpointer	 data);
+gboolean    g_ptr_array_remove_fast        (GPtrArray	*array,
+					    gpointer	 data);
 void	    g_ptr_array_add		   (GPtrArray	*array,
 					    gpointer	 data);
 
@@ -1597,17 +1688,21 @@ void	    g_ptr_array_add		   (GPtrArray	*array,
  * but type-safe.
  */
 
-GByteArray* g_byte_array_new	  (void);
-void	    g_byte_array_free	  (GByteArray	*array,
-				   gboolean	 free_segment);
-GByteArray* g_byte_array_append	  (GByteArray	*array,
-				   const guint8 *data,
-				   guint	 len);
-GByteArray* g_byte_array_prepend  (GByteArray	*array,
-				   const guint8 *data,
-				   guint	 len);
-GByteArray* g_byte_array_set_size (GByteArray	*array,
-				   guint	 length);
+GByteArray* g_byte_array_new	           (void);
+void	    g_byte_array_free	           (GByteArray	 *array,
+					    gboolean	  free_segment);
+GByteArray* g_byte_array_append	           (GByteArray	 *array,
+					    const guint8 *data,
+					    guint	  len);
+GByteArray* g_byte_array_prepend           (GByteArray	 *array,
+					    const guint8 *data,
+					    guint	  len);
+GByteArray* g_byte_array_set_size          (GByteArray	 *array,
+					    guint	  length);
+GByteArray* g_byte_array_remove_index	   (GByteArray	 *array,
+					    guint	  index);
+GByteArray* g_byte_array_remove_index_fast (GByteArray	 *array,
+					    guint	  index);
 
 
 /* Hash Functions
@@ -1860,6 +1955,7 @@ GScanner*	g_scanner_new			(GScannerConfig *config_templ);
 void		g_scanner_destroy		(GScanner	*scanner);
 void		g_scanner_input_file		(GScanner	*scanner,
 						 gint		input_fd);
+void		g_scanner_sync_file_offset	(GScanner	*scanner);
 void		g_scanner_input_text		(GScanner	*scanner,
 						 const	gchar	*text,
 						 guint		text_len);
@@ -1996,14 +2092,108 @@ gpointer   g_tuples_index     (GTuples	   *tuples,
  * next largest prime, or the highest it knows about which is about
  * MAXINT/4.
  */
-
 guint	   g_spaced_primes_closest (guint num);
 
-/* Glib version.
+
+/* IO Channels.
+ * These are used for plug-in communication in the GIMP, for instance.
+ * On Unix, it's simply an encapsulated file descriptor (a pipe).
+ * On Windows, it's a handle to an anonymouos pipe, *and* (in the case
+ * of the writable end) a thread id to post a message to when you have written
+ * stuff.
  */
-extern const guint glib_major_version;
-extern const guint glib_minor_version;
-extern const guint glib_micro_version;
+struct _GIOChannel
+{
+  gint fd;			/* file handle (pseudo such in Win32) */
+#ifdef NATIVE_WIN32
+  guint peer;			/* thread to post message to */
+  guint peer_fd;		/* read handle (in the other process) */
+  guint offset;			/* counter of accumulated bytes, to
+				 * be included in the message posted
+				 * so we keep in sync.
+				 */
+  guint peer_offset;		/* in input channels where the writer's
+				 * offset is, so we don't try to read too much
+				 */
+#endif
+};
+
+GIOChannel *g_iochannel_new            (gint        fd);
+void        g_iochannel_free           (GIOChannel *channel);
+void        g_iochannel_close_and_free (GIOChannel *channel);
+void        g_iochannel_wakeup_peer    (GIOChannel *channel);
+#ifndef NATIVE_WIN32
+#  define   g_iochannel_wakeup_peer(channel) G_STMT_START { } G_STMT_END
+#endif
+
+
+/* Windows emulation stubs for common unix functions
+ */
+#ifdef NATIVE_WIN32
+
+#define MAXPATHLEN 1024
+
+#ifdef _MSC_VER
+typedef int pid_t;
+
+/* These POSIXish functions are available in the Microsoft C library
+ * prefixed with underscore (which of course technically speaking is
+ * the Right Thing, as they are non-ANSI. Not that being non-ANSI
+ * prevents Microsoft from practically requiring you to include
+ * <windows.h> every now and then...).
+ *
+ * You still need to include the appropriate headers to get the
+ * prototypes, <io.h> or <direct.h>.
+ *
+ * For some functions, we provide emulators in glib, which are prefixed
+ * with gwin_.
+ */
+#define getcwd			_getcwd
+#define getpid			_getpid
+#define access			_access
+#define open			_open
+#define read			_read
+#define write			_write
+#define lseek			_lseek
+#define close			_close
+#define pipe(phandles)		_pipe (phandles, 4096, _O_BINARY)
+#define popen			_popen
+#define pclose			_pclose
+#define fdopen			_fdopen
+#define ftruncate(fd, size)	gwin_ftruncate (fd, size)
+#define opendir			gwin_opendir
+#define readdir			gwin_readdir
+#define rewinddir		gwin_rewinddir
+#define closedir		gwin_closedir
+
+#define NAME_MAX 255
+
+struct DIR
+{
+  gchar    *dir_name;
+
+  gboolean  just_opened;
+  guint     find_file_handle;
+  gpointer  find_file_data;
+};
+typedef struct DIR DIR;
+struct dirent
+{
+  gchar  d_name[NAME_MAX + 1];
+};
+
+/* emulation functions */
+extern int	gwin_ftruncate	(gint		 f,
+				 guint		 size);
+DIR*		gwin_opendir	(const gchar	*dirname);
+struct dirent*	gwin_readdir  	(DIR		*dir);
+void		gwin_rewinddir 	(DIR		*dir);
+gint		gwin_closedir  	(DIR		*dir);
+
+#endif /* _MSC_VER */
+
+#endif /* NATIVE_WIN32 */
+
 
 #ifdef __cplusplus
 }
