@@ -21,17 +21,24 @@
  */
 #define		__gscanner_c__
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include	<stdlib.h>
 #include	<stdarg.h>
 #include	<string.h>
 #include	<stdio.h>
+#include	"glib.h"
+#ifdef HAVE_UNISTD_H
 #include	<unistd.h>
+#endif
 #include	<errno.h>
 #include	<sys/types.h>	/* needed for sys/stat.h */
 #include	<sys/stat.h>
-#include	"glib.h"
-
-
+#ifdef _MSC_VER
+#include	<io.h>		/* For _read() */
+#endif
 
 /* --- defines --- */
 #define	to_lower(c)				( \
@@ -61,7 +68,7 @@ struct	_GScannerKey
 static GScannerConfig g_scanner_config_template =
 {
   (
-   " \t\n"
+   " \t\r\n"
    )			/* cset_skip_characters */,
   (
    G_CSET_a_2_z
@@ -613,10 +620,15 @@ g_scanner_cur_value (GScanner *scanner)
 {
   GTokenValue v;
   
-  v.v_int = 0;
   g_return_val_if_fail (scanner != NULL, v);
+
+  /* MSC isn't capable of handling return scanner->value; ? */
+
+  v.v_int = 0;
   
-  return scanner->value;
+  v = scanner->value;
+
+  return v;
 }
 
 guint
@@ -650,6 +662,9 @@ g_scanner_input_file (GScanner *scanner,
   g_return_if_fail (scanner != NULL);
   g_return_if_fail (input_fd >= 0);
 
+  if (scanner->input_fd >= 0)
+    g_scanner_sync_file_offset (scanner);
+
   scanner->token = G_TOKEN_NONE;
   scanner->value.v_int = 0;
   scanner->line = 1;
@@ -675,6 +690,9 @@ g_scanner_input_text (GScanner	  *scanner,
   else
     text = NULL;
 
+  if (scanner->input_fd >= 0)
+    g_scanner_sync_file_offset (scanner);
+
   scanner->token = G_TOKEN_NONE;
   scanner->value.v_int = 0;
   scanner->line = 1;
@@ -729,6 +747,33 @@ g_scanner_peek_next_char (GScanner *scanner)
     return 0;
 }
 
+void
+g_scanner_sync_file_offset (GScanner *scanner)
+{
+  g_return_if_fail (scanner != NULL);
+
+  /* for file input, rewind the filedescriptor to the current
+   * buffer position and blow the file read ahead buffer. usefull for
+   * third party uses of our filedescriptor, which hooks onto the current
+   * scanning position.
+   */
+
+  if (scanner->input_fd >= 0 && scanner->text_end > scanner->text)
+    {
+      gint buffered;
+
+      buffered = scanner->text_end - scanner->text;
+      if (lseek (scanner->input_fd, - buffered, SEEK_CUR) >= 0)
+	{
+	  /* we succeeded, blow our buffer's contents now */
+	  scanner->text = NULL;
+	  scanner->text_end = NULL;
+	}
+      else
+	errno = 0;
+    }
+}
+
 static guchar
 g_scanner_get_char (GScanner	*scanner,
 		    guint	*line_p,
@@ -760,6 +805,12 @@ g_scanner_get_char (GScanner	*scanner,
 	  scanner->text = buffer + 1;
 	  scanner->text_end = buffer + count;
 	  fchar = *buffer;
+	  if (!fchar)
+	    {
+	      g_scanner_sync_file_offset (scanner);
+	      scanner->text_end = scanner->text;
+	      scanner->input_fd = -1;
+	    }
 	}
     }
   else
@@ -1064,9 +1115,11 @@ g_scanner_stat_mode (const gchar *filename)
   gint		st_mode;
   
   stat_buf = g_new0 (struct stat, 1);
-  
+#ifdef HAVE_LSTAT  
   lstat (filename, stat_buf);
-  
+#else
+  stat (filename, stat_buf);
+#endif
   st_mode = stat_buf->st_mode;
   
   g_free (stat_buf);
