@@ -16,6 +16,11 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -823,11 +828,11 @@ g_printf_string_upper_bound (const gchar* format,
 		case 'u':
 		case 'x':
 		case 'X':
-#ifdef	HAVE_GINT64
+#ifdef	G_HAVE_GINT64
 		  if (extra_long)
 		    (void) va_arg (args, gint64);
 		  else
-#endif	/* HAVE_GINT64 */
+#endif	/* G_HAVE_GINT64 */
 		    {
 		      if (long_int)
 			(void) va_arg (args, long);
@@ -951,6 +956,9 @@ g_strcasecmp (const gchar *s1,
 #else
   gint c1, c2;
   
+  g_return_val_if_fail (s1 != NULL, 0);
+  g_return_val_if_fail (s2 != NULL, 0);
+
   while (*s1 && *s2)
     {
       /* According to A. Cox, some platforms have islower's that
@@ -967,6 +975,38 @@ g_strcasecmp (const gchar *s1,
 #endif
 }
 
+gint
+g_strncasecmp (const gchar *s1,
+	       const gchar *s2,
+	       guint n)
+{
+#ifdef HAVE_STRNCASECMP
+  return strncasecmp (s1, s2, n);
+#else
+  gint c1, c2;
+  
+  g_return_val_if_fail (s1 != NULL, 0);
+  g_return_val_if_fail (s2 != NULL, 0);
+
+  while (n-- && *s1 && *s2)
+    {
+      /* According to A. Cox, some platforms have islower's that
+       * don't work right on non-uppercase
+       */
+      c1 = isupper ((guchar)*s1) ? tolower ((guchar)*s1) : *s1;
+      c2 = isupper ((guchar)*s2) ? tolower ((guchar)*s2) : *s2;
+      if (c1 != c2)
+	return (c1 - c2);
+      s1++; s2++;
+    }
+
+  if (n)
+    return (((gint)(guchar) *s1) - ((gint)(guchar) *s2));
+  else
+    return 0;
+#endif
+}
+
 gchar*
 g_strdelimit (gchar	  *string,
 	      const gchar *delimiters,
@@ -988,6 +1028,38 @@ g_strdelimit (gchar	  *string,
   return string;
 }
 
+gchar*
+g_strescape (gchar *string)
+{
+  gchar *q;
+  gchar *escaped;
+  guint backslashes = 0;
+  gchar *p = string;
+
+  g_return_val_if_fail (string != NULL, NULL);
+
+  while (*p != '\000')
+    backslashes += (*p++ == '\\');
+
+  if (!backslashes)
+    return g_strdup (string);
+
+  escaped = g_new (gchar, strlen (string) + backslashes + 1);
+
+  p = string;
+  q = escaped;
+
+  while (*p != '\000')
+    {
+      if (*p == '\\')
+	*q++ = '\\';
+      *q++ = *p++;
+    }
+  *q = '\000';
+
+  return escaped;
+}
+
 /* blame Elliot for these next five routines */
 gchar*
 g_strchug (gchar *string)
